SystemOrganization addCategory: #'Neo-JSON-Tests'!

Object subclass: #NeoJSONBenchmark
	instanceVariableNames: 'objects json'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONBenchmark commentStamp: 'SvenVanCaekenberghe 6/5/2012 09:48' prior: 0!
I am NeoJSONBenchmark.
I hold 1000 objects of the form Dictionary('color'->'green' 'enabled'->false 'id'->1 'name'->'Name-1' 'value'->0.002 ) in an Array.
I hold the JSON serialization of that as a String of the form '[{"value":0.002,"enabled":false,"name":"Name-1","color":"green","id":1}, ...

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark read] bench.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark write] bench.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark read] timeToRun.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark write] timeToRun.!

!NeoJSONBenchmark methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/4/2012 18:49'!
initialize
	super initialize.
	objects := Array new: 1000.
	1 to: 1000 do: [ :each |
		| map |
		(map := Dictionary new)
			at: 'id' put: each;
			at: 'name' put: 'Name-', each asString;
			at: 'enabled' put: (#( true false) at: (each \\ 2) + 1);
			at: 'color' put: (#( 'red' 'green' 'blue' ) at: (each \\ 3) + 1);
			at: 'value' put: each + each / 1000.0.
		objects at: each put: map ].
	json := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: objects ]
		! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/4/2012 17:02'!
read
	^ (NeoJSONReader on: json readStream) next
! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/4/2012 17:26'!
write
	^ String streamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: objects ]
! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/6/2012 15:07'!
writePrettyPrintedRead
	| output |
	output := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) 
			prettyPrint: true;
			nextPut: objects ].
	^ (NeoJSONReader on: output readStream) 
		next! !

Object subclass: #NeoJSONMockStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONMockStream commentStamp: '<historical>' prior: 0!
I am NeoJSONMockStream.
I wrap another stream to validate the limited API used.!

!NeoJSONMockStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/13/2012 16:25'!
on: stream
	^ self new
		stream: stream;
		yourself! !

!NeoJSONMockStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:23'!
atEnd
	^ stream atEnd! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:23'!
close
	^ stream close! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:22'!
next
	^ stream next! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:23'!
nextPut: character
	^ stream nextPut: character! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:23'!
nextPutAll: string
	^ stream nextPutAll: string! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:22'!
peek
	^ stream peek! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2012 10:53'!
peekFor: char
	^ stream peekFor: char! !

!NeoJSONMockStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 16:23'!
space
	^ stream space! !

!NeoJSONMockStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 16:22'!
stream: anObject
	stream := anObject! !

Object subclass: #NeoJSONTestObject1
	instanceVariableNames: 'id name timestamp points bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONTestObject1 commentStamp: '<historical>' prior: 0!
I am NeoJSONTestObject1.

I repesent a domain model object.
I have the following properties:

- id <Integer>
- name <String>
- timestamp <DateAndTime>
- points <Array of: Point>
- bytes <ByteArray>

My JSON representation is like this:

- id <Number>
- name <String>
- created-at <ISO8601 String>
- points <Array of: <Map x,y>
- bytes <Array of: Number>
!

!NeoJSONTestObject1 class methodsFor: 'instance creation' stamp: 'marianopeck 1/30/2018 13:07'!
example1
	^ self new
		id: 123;
		name: 'Example1';
		timestamp: (DateAndTime year: 2012 month: 06 day: 08 hour: 14 minute: 30 second: 15 offset: 1 hour);
		points: (Array with: 1@2 with: 3@4);
		bytes: #[ 2 4 6 8 ];
		yourself! !

!NeoJSONTestObject1 class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2012 12:34'!
neoJsonMapping: mapper
	mapper for: self do: [ :mapping |
		mapping mapInstVars: #(id name).
		(mapping mapInstVar: #timestamp to: 'created-at') valueSchema: DateAndTime.
		(mapping mapInstVar: #points) valueSchema: #ArrayOfPoints.
		(mapping mapInstVar: #bytes) valueSchema: ByteArray ].
	mapper for: DateAndTime customDo: [ :mapping |
		mapping decoder: [ :string | DateAndTime fromString: string ].
		mapping encoder: [ :dateAndTime | dateAndTime printString ] ].
	mapper for: #ArrayOfPoints customDo: [ :mapping |
		mapping listOfElementSchema: Point ].  
	mapper mapAllInstVarsFor: Point.
	mapper for: ByteArray customDo: [ :mapping |
		mapping listOfType: ByteArray ]! !

!NeoJSONTestObject1 methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:45'!
= anObject
	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ id = anObject id
		and: [
			name = anObject name
				and: [
					timestamp = anObject timestamp
						and: [
							points = anObject points
								and: [ bytes = anObject bytes ] ] ] ]! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
bytes
	^ bytes! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:04'!
bytes: byteArray
	bytes := byteArray! !

!NeoJSONTestObject1 methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:46'!
hash
	^ id hash bitXor: (name hash bitXor: (timestamp hash bitXor: (points hash bitXor: bytes hash)))! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:02'!
id
	^ id! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
id: integer
	id := integer! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:02'!
name
	^ name! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
name: string
	name := string! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
points
	^ points! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:04'!
points: arrayOfPoints
	points := arrayOfPoints ! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
timestamp
	^ timestamp! !

!NeoJSONTestObject1 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:03'!
timestamp: dateAndTime 
	timestamp := dateAndTime ! !

Object subclass: #NeoJSONTestObject2
	instanceVariableNames: 'id width height data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONTestObject2 commentStamp: 'SvenVanCaekenberghe 3/22/2014 09:36' prior: 0!
I am NeoJSONTestObject2.

Instance Variables
	data:		<String>
	height:		<Integer>
	id:			<Integer>
	width:		<Integer>

Used for benchmarking and other tests.!

!NeoJSONTestObject2 class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:53'!
benchmark10k
	"self benchmark10k"
	
	self benchmark: 10000! !

!NeoJSONTestObject2 class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:54'!
benchmark10kAsDictionary
	"self benchmark10kAsDictionary"
	
	self benchmarkAsDictionary: 10000! !

!NeoJSONTestObject2 class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:54'!
benchmark200k
	"self benchmark200k"
	
	self benchmark: 200000! !

!NeoJSONTestObject2 class methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:09'!
benchmark: count
	"self benchmark: 1000"
	
	| collection json size |
	collection := Array new: count withAll: self example1.
	size := 2 + (count * (NeoJSONWriter toString: self example1) size) + count - 1.
	Transcript 
		cr;
		<< ('Serializing {1} objects took ms ' format: (Array with: count) );
		show: [ json := String new: size streamContents: [ :stream |
								(NeoJSONWriter on: stream) nextPut: collection ] ] timeToRun.
	Transcript 
		cr;
		<< ('Deserializing {1} objects took ms ' format: (Array with: count) );
		show: [ NeoJSONReader fromString: json ] timeToRun.! !

!NeoJSONTestObject2 class methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:09'!
benchmarkAsDictionary: count
	"self benchmarkAsDictionary: 1000"
	
	| collection json size |
	collection := Array new: count withAll: self example1 asDictionary.
	size := 2 + (count * (NeoJSONWriter toString: self example1 asDictionary) size) + count - 1.
	Transcript 
		cr;
		<< ('Serializing {1} objects took ms ' format: (Array with: count) );
		show: [ json := String new: size streamContents: [ :stream |
								(NeoJSONWriter on: stream) nextPut: collection ] ] timeToRun.
	Transcript 
		cr;
		<< ('Deserializing {1} objects took ms ' format: (Array with: count) );
		show: [ NeoJSONReader fromString: json ] timeToRun.! !

!NeoJSONTestObject2 class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
example1
	^ self new
		id: 13;
		width: 250;
		height: 110;
		data: (String new: 512 withAll: $a);
		yourself! !

!NeoJSONTestObject2 class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 10:50'!
neoJsonMapping: mapper
	"For inheritance mapping to work, the schema/classname must be self,
	but the instance variables have to be enumerated manually per class"

	mapper for: self do: [ :mapping |
		mapping mapInstVars: #(id width height data) ] ! !

!NeoJSONTestObject2 methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:22'!
= anObject
	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ id = anObject id
		and: [
			height = anObject height
				and: [
					width = anObject width
						and: [
							data = anObject data ] ] ]! !

!NeoJSONTestObject2 methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/10/2012 20:47'!
asDictionary
	^ Dictionary new
		at: #id put: id;
		at: #height put: height;
		at: #width put: width;
		at: #data put: data;
		yourself! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
data
	^ data! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
data: anObject
	data := anObject! !

!NeoJSONTestObject2 methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:22'!
hash
	^ id hash bitXor: (width hash bitXor: (height hash bitXor: data hash))! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
height
	^ height! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
height: anObject
	height := anObject! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
id
	^ id! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
id: anObject
	id := anObject! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
width
	^ width! !

!NeoJSONTestObject2 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:04'!
width: anObject
	width := anObject! !

NeoJSONTestObject2 subclass: #NeoJSONTestObject3
	instanceVariableNames: 'color transparent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONTestObject3 commentStamp: 'SvenVanCaekenberghe 3/22/2014 09:40' prior: 0!
I am NeoJSONTestObject3, a subclass of NeoJSONTestObject2.

Instance Variables
	color:			<String>
	transparent :	<Boolean>

I am used for inheritance testing.
!

!NeoJSONTestObject3 class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/22/2014 09:43'!
example1
	^ super example1
		data: (String new: 8 withAll: $a);
		color: 'red';
		transparent: true;
		yourself! !

!NeoJSONTestObject3 class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 10:50'!
neoJsonMapping: mapper
	"For inheritance mapping to work, the schema/classname must be self,
	but the instance variables have to be enumerated manually per class"
	
	super neoJsonMapping: mapper.
	mapper for: self do: [ :mapping |
		mapping mapInstVars: #(color transparent) ]! !

!NeoJSONTestObject3 methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 3/22/2014 09:45'!
= anObject
	^ super = anObject 
		and: [ color = anObject color
			and: [ transparent = anObject transparent ] ]! !

!NeoJSONTestObject3 methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 3/22/2014 09:46'!
asDictionary
	^ super asDictionary
		at: #color put: color;
		at: #transparent put: transparent;
		yourself! !

!NeoJSONTestObject3 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 09:43'!
color
	^ color! !

!NeoJSONTestObject3 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 09:43'!
color: anObject
	color := anObject! !

!NeoJSONTestObject3 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 09:43'!
transparent
	^ transparent! !

!NeoJSONTestObject3 methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/22/2014 09:43'!
transparent: anObject
	transparent := anObject! !

Object subclass: #NeoJSONValidation
	instanceVariableNames: 'gitDirectory logStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONValidation commentStamp: 'SvenVanCaekenberghe 5/28/2017 16:40' prior: 0!
I am NeoJSONValidation, a set of tests to validate NeoJSON against some external rules.

Both positive & negative tests are includes.
Some of these are implementation choices.

http://seriot.ch/parsing_json.php
https://github.com/nst/JSONTestSuite
https://github.com/nst/JSONTestSuite.git
  test_parsing
  test_transform

https://github.com/miloyip/nativejson-benchmark
https://github.com/miloyip/nativejson-benchmark.git
  data/jsonchecker
  data/roundtrip

This can only be run with the correct setup (checkout/config).

  NeoJSONValidation new runNativeJsonBenchmarkRoundTrip.
  NeoJSONValidation new runNativeJsonBenchmarkJsonCheckerPasses.
  NeoJSONValidation new runNativeJsonBenchmarkJsonCheckerFailures.

  NeoJSONValidation new runJsonTestSuiteTestParsingPasses.
  NeoJSONValidation new runJsonTestSuiteTestParsingFailures.
  NeoJSONValidation new runJsonTestSuiteTestParsingInfo.

Logging goes to a file, neo-json-validation.log!

!NeoJSONValidation methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/29/2017 09:28'!
closeLog
	logStream ifNotNil: [ 
		logStream close.
		logStream := nil ].
	^ self logFile exists
		ifTrue: [ self logFile contents ] 
		ifFalse: [ 'log file is empty' ]! !

!NeoJSONValidation methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/29/2017 09:27'!
ensureLog
	logStream ifNil: [ 
		logStream := self logFile ensureDelete; writeStream. 
		logStream setToEnd ]
	! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:25'!
gitDirectory
	^ gitDirectory ifNil: [ gitDirectory := FileLocator home / #Develop / #git ]! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:23'!
gitDirectory: fileReference
	gitDirectory := fileReference! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:26'!
jsonTestSuiteDirectory
	^ self gitDirectory / #JSONTestSuite! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:27'!
jsonTestSuiteTestParsingDirectory
	^ self jsonTestSuiteDirectory / 'test_parsing'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 13:50'!
jsonTestSuiteTestParsingFailFiles
	^ self jsonTestSuiteTestParsingDirectory childrenMatching: 'n_*.json'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 13:50'!
jsonTestSuiteTestParsingInfoFiles
	^ self jsonTestSuiteTestParsingDirectory childrenMatching: 'i_*.json'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 13:50'!
jsonTestSuiteTestParsingPassFiles
	^ self jsonTestSuiteTestParsingDirectory childrenMatching: 'y_*.json'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:27'!
jsonTestSuiteTestTransformDirectory
	^ self jsonTestSuiteDirectory / 'test_transform'! !

!NeoJSONValidation methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/29/2017 09:25'!
log: message
	self ensureLog.
	logStream nextPutAll: message; lf; flush
! !

!NeoJSONValidation methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/29/2017 09:26'!
logFile
	^ 'neo-json-validation.log' asFileReference! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:28'!
nativeJsonBenchmarkDataDirectory
	^ self nativeJsonBenchmarkDirectory / #data! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:28'!
nativeJsonBenchmarkDirectory
	^ self gitDirectory / 'nativejson-benchmark'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:29'!
nativeJsonBenchmarkJsonCheckerDirectory
	^ self nativeJsonBenchmarkDataDirectory / #jsonchecker! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 12:05'!
nativeJsonBenchmarkJsonCheckerFailFiles
	^ self nativeJsonBenchmarkJsonCheckerDirectory childrenMatching: 'fail*.json'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 12:05'!
nativeJsonBenchmarkJsonCheckerPassFiles
	^ self nativeJsonBenchmarkJsonCheckerDirectory childrenMatching: 'pass*.json'! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:29'!
nativeJsonBenchmarkRoundTripDirectory
	^ self nativeJsonBenchmarkDataDirectory / #roundtrip! !

!NeoJSONValidation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2017 11:34'!
nativeJsonBenchmarkRoundTripFiles
	^ self nativeJsonBenchmarkRoundTripDirectory children! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:57'!
runJsonTestSuiteTestParsingFailures
	^ self
		validateFiles: self jsonTestSuiteTestParsingFailFiles 
		usingTest: [ :file | 
			| data success |
			success := false.
			[ data := file readStreamDo: [ :in | (NeoJSONReader on: in) next; failIfNotAtEnd ] ]
				on: NeoJSONParseError
				do: [ :exception | self log: exception printString. success := true ].
			success ]! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:25'!
runJsonTestSuiteTestParsingInfo
	^ self
		validateFiles: self jsonTestSuiteTestParsingInfoFiles 
		usingTest: [ :file | 
			| data success |
			success := true.
			[ data := file readStreamDo: [ :in | (NeoJSONReader on: in) next ] ]
				on: NeoJSONParseError
				do: [ :exception | self log: exception printString. success := false ].
			success ]! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:25'!
runJsonTestSuiteTestParsingPasses
	^ self
		validateFiles: self jsonTestSuiteTestParsingPassFiles 
		usingTest: [ :file | 
			| data success |
			success := true.
			[ data := file readStreamDo: [ :in | (NeoJSONReader on: in) next ] ]
				on: NeoJSONParseError
				do: [ :exception | self log: exception printString. success := false ].
			success ]! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:40'!
runNativeJsonBenchmarkJsonCheckerFailures
	^ self
		validateFiles: self nativeJsonBenchmarkJsonCheckerFailFiles
		usingTest: [ :file | 
			| data success |
			success := false.
			[ data := file readStreamDo: [ :in | (NeoJSONReader on: in) next; failIfNotAtEnd ] ]
				on: NeoJSONParseError
				do: [ :exception | self log: exception printString. success := true ].
			success ]! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:25'!
runNativeJsonBenchmarkJsonCheckerPasses
	^ self
		validateFiles: self nativeJsonBenchmarkJsonCheckerPassFiles
		usingTest: [ :file | 
			| data success |
			success := true.
			[ data := file readStreamDo: [ :in | (NeoJSONReader on: in) next ] ]
				on: NeoJSONParseError
				do: [ :exception | self log: exception printString. success := false ].
			success ]! !

!NeoJSONValidation methodsFor: 'running' stamp: 'SvenVanCaekenberghe 5/29/2017 09:25'!
runNativeJsonBenchmarkRoundTrip
	^ self 
		validateFiles: self nativeJsonBenchmarkRoundTripFiles
		usingTest: [ :file |
			| data json |
			data := file readStreamDo: [ :in | (NeoJSONReader on: in) next ].
			json := NeoJSONWriter toString: data.
			json = file contents ]! !

!NeoJSONValidation methodsFor: 'private' stamp: 'marianopeck 1/30/2018 13:21'!
validateFiles: files usingTest: block
	| total succeeded failed |
	total := succeeded := failed := 0.
	files do: [ :file | 
		| success |
		self log: 'Running ' , file pathString.
		self log: ([ file contents contractTo: 64 ] on: Error do: [ :exception | exception printString ]).
		success := [ block value: file ]
			on: Error
			do: [ :exception | self log: '  Errored. ' , exception printString. false ].
		total := total + 1.
		success
			ifTrue: [ succeeded := succeeded + 1 ]
			ifFalse: [ failed := failed + 1 ].
		self log: (success ifTrue: [ '  Succeeded' ] ifFalse: [ '  Failed' ]); log: '' ].
	self log: ('{1} total = {2} succeeded + {3} failed' format: (Array with: total with: succeeded with: failed) ).
	^ self closeLog! !

TestCase subclass: #NeoJSONBenchmarkTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONBenchmarkTests commentStamp: 'SvenVanCaekenberghe 6/5/2012 10:05' prior: 0!
I am NeoJSONBenchmarkTests.

This test is obviously dependent on hardware, but the margin is quite high.!

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/10/2012 20:20'!
testObject2
	| json |
	json := NeoJSONWriter toString: NeoJSONTestObject2 new.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject2)
		equals: NeoJSONTestObject2 new! !

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/15/2014 19:29'!
testPrettyPrinted
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	self
		assert: benchmark writePrettyPrintedRead
		equals: benchmark read! !

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/15/2014 19:30'!
testRead
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	benchmark read! !

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/15/2014 19:30'!
testWrite
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	benchmark write! !

TestCase subclass: #NeoJSONExamplesTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONExamplesTests commentStamp: 'SvenVanCaekenberghe 7/6/2017 09:50' prior: 0!
NeoJSONExamplesTests runs a couple of examples as tests!

!NeoJSONExamplesTests methodsFor: 'tests' stamp: 'marianopeck 1/30/2018 13:05'!
testPoints
	| points json result |
	
	points := Array with: 1@2 with: 3@4 with: 5@6.
	
	json := String streamContents: [ :out |
		(NeoJSONWriter on: out)
			prettyPrint: true;
			mapInstVarsFor: Point;
			nextPut: points ].
		
	result := (NeoJSONReader on: json readStream)
		mapInstVarsFor: Point;
		for: #ListOfPoints customDo: [ :mapping |
			mapping listOfElementSchema: Point ];
		nextAs: #ListOfPoints.
	
	self assert: result equals: points.
			
	result := (NeoJSONReader on: json readStream)
		mapInstVarsFor: Point;
		nextListAs: Point.
	
	self assert: result equals: points.
		
	result := (NeoJSONReader on: json readStream)
		mapInstVarsFor: Point;
		for: #OrderedCollectionOfPoints customDo: [ :mapping |
			mapping listOfType: OrderedCollection andElementSchema: Point ];
		nextAs: #OrderedCollectionOfPoints.
	
	self assert: result equals: points asOrderedCollection.! !

!NeoJSONExamplesTests methodsFor: 'tests' stamp: 'SvenVanCaekenberghe 7/6/2017 09:56'!
testRectanglePoints
	| rectangle rectangleJson result |

	rectangle := Rectangle origin: 3 @ 4 extent: 5 @ 6.

	rectangleJson := String streamContents: [ :stream | 
		(NeoJSONWriter on: stream)
			prettyPrint: true;
			mapInstVarsFor: Point;
			mapInstVarsFor: Rectangle;
			nextPut: rectangle ].

	result := (NeoJSONReader on: rectangleJson readStream)
		mapInstVarsFor: Point;
		for: Rectangle do: [ :mapping | 
			(mapping mapInstVar: #origin) valueSchema: Point.
			(mapping mapInstVar: #corner) valueSchema: Point ];
		nextAs: Rectangle.

	self assert: result equals: rectangle.

	result := (NeoJSONReader on: rectangleJson readStream)
		mapInstVarsFor: Point;
		for: Rectangle do: [ :mapping | 
			mapping mapInstVars do: [ :each | each valueSchema: Point ] ];
		nextAs: Rectangle.
		
	self assert: result equals: rectangle ! !

!NeoJSONExamplesTests methodsFor: 'tests' stamp: 'SvenVanCaekenberghe 11/13/2017 18:51'!
testRectanglePointsWithNils
	self
		assert: 
			(String streamContents: [ :stream | 
				(NeoJSONWriter on: stream)
					for: Point do: [ :mapping | mapping mapAllInstVars ];
					for: Rectangle do: [ :mapping | 
						(mapping mapInstVar: #origin) valueSchema: Point.
						(mapping mapInstVar: #corner) valueSchema: Point ];
					nextPut: Rectangle new ])
		equals: '{}'.
	self
		assert:
			(String streamContents: [ :stream | 
				(NeoJSONWriter on: stream)
					for: Point do: [ :mapping | mapping mapAllInstVars ];
					for: Rectangle	do: [ :mapping | 
						(mapping mapInstVar: #origin) valueSchema: Point.
						(mapping mapInstVar: #corner) valueSchema: Point ];
					writeNil: true;
					nextPut: Rectangle new ])
		equals: '{"origin":null,"corner":null}'.! !

TestCase subclass: #NeoJSONMappingTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONMappingTests commentStamp: 'SvenVanCaekenberghe 9/22/2017 15:07' prior: 0!
I am NeoJSONMappingTests, testing more exotic kinds of mapping constructs.!

!NeoJSONMappingTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 09:05'!
testDynamicTyping
	| data customMapping json result |
	data := Array with: 'foo'->1 with: #('foo' 2).
	"The idea is to map a key value combination as either a classic association or a simple pair, 
	using key & value properties as well as a type property to distinguish between the two"
	customMapping := [ :mapper |
		mapper 
			for: #AssocOrPair customDo: [ :mapping |
				mapping
					encoder: [ :x | 
						x isArray 
							ifTrue: [ Dictionary new at: 'type' put: 'pair'; at: 'key' put: x first; at: 'value' put: x second; yourself ] 
							ifFalse: [ Dictionary new at: 'type' put: 'assoc'; at: 'key' put: x key; at: 'value' put: x value; yourself ] ];
					decoder: [ :x |
						(x at: 'type') = 'pair'
							ifTrue: [ Array with: (x at: 'key') with: (x at: 'value') ]
							ifFalse: [ (x at: 'key') -> (x at: 'value')] ] ];
			for: #ArrayOfAssocOrPair customDo: [ :mapping |
				mapping listOfType: Array andElementSchema: #AssocOrPair ];
			yourself ].
	json := String streamContents: [ :out |
		(customMapping value: (NeoJSONWriter on: out)) nextPut: data as: #ArrayOfAssocOrPair ].
	result := (customMapping value: (NeoJSONReader on: json readStream)) nextAs: #ArrayOfAssocOrPair.
	self assert: result equals: data! !

!NeoJSONMappingTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/6/2018 17:38'!
testSymbolOrStringInstVarMapping
	| data out1 out2 result1 result2 |
	data := Array with: 1@2 with: 3@4.
	out1 := String streamContents: [ :out |
		(NeoJSONWriter on: out)
			for: Point do: [ :mapping | mapping mapInstVars: #(x y) ];
			nextPut: data ].
	result1 := (NeoJSONReader on: out1 readStream)
		for: Point do: [ :mapping | mapping mapInstVars: #(x y) ];
		nextListAs: Point.
	self assert: result1 equals: data.
	out2 := String streamContents: [ :out |
		(NeoJSONWriter on: out)
			for: Point do: [ :mapping | mapping mapInstVars: #('x' 'y') ];
			nextPut: data ].
	result2 := (NeoJSONReader on: out1 readStream)
		for: Point do: [ :mapping | mapping mapInstVars: #('x' 'y') ];
		nextListAs: Point.
	self assert: result2 equals: data! !

!NeoJSONMappingTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:07'!
testVirtualTransferObject
	| data customMapping extraJson json result |
	data := NeoJSONObject new data: (Array with: 1@1 with: 2@2).
	extraJson := '{"foo":1,"data":[{"x":1,"y":1},{"x":2,"y":2,"z":-1}]}'.
	"The idea here is that we are not interested in the top level object just in its data property, 
	which should be of a specific type, hence we create a virtual transfer object"
	customMapping := [ :mapper |
		mapper
			for: Point do: [ :mapping |
				mapping mapInstVars: #(x y) ];
			for: #ArrayOfPoints customDo: [ :mapping |
				mapping listOfElementSchema: Point ];
			for: #TransferObject do: [ :mapping |
				mapping subjectClass: NeoJSONObject.
				(mapping mapAccessor: #data) valueSchema: #ArrayOfPoints ];
			yourself ].
	"By using NeoJSONObject accessors (#data & #data:) are translated to generic #at:[put:] messages.
	A longer alternative is 
	(mapping mapProperty: #data getter: [ :obj | obj at: #data ] setter: [ :obj :x | obj at: #data put: x]) valueSchema: #ArrayOfPoints
	where the blocks give you the flexibility to use a plain Dictionary for example"
	result := (customMapping value: (NeoJSONReader on: extraJson readStream)) nextAs: #TransferObject.
	self assert: result equals: data.
	json := String streamContents: [ :out |
		(customMapping value: (NeoJSONWriter on: out)) nextPut: data as: #TransferObject ].
	result := (customMapping value: (NeoJSONReader on: json readStream)) nextAs: #TransferObject.
	self assert: result equals: data! !

TestCase subclass: #NeoJSONObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONObjectTests commentStamp: 'SvenVanCaekenberghe 9/8/2014 12:39' prior: 0!
I am NeoJSONObjectTests, I hold unit tests for NeoJSONObject.
!

!NeoJSONObjectTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 11:13'!
testAtPath
	| object |
	object := NeoJSONObject newFrom:  (Array with:  #one -> ( NeoJSONObject newFrom:  (Array with:  #two -> ( NeoJSONObject newFrom: (Array with:  #three -> 42) ) ) )).
	self assert: (object atPath: #(one two three)) equals: 42.
	self assert: (object atPath: #(one two three)) equals: object one two three.	
	self assert: (object atPath: #(missing)) equals: nil.
	self assert: (object atPath: #(one missing)) equals: nil.
	self assert: (object atPath: #(one two missing)) equals: nil.
	self assert: (object atPath: #(one two missing)) equals: object one two missing.
	
	object := NeoJSONObject newFrom:  (Array with: #foo -> 42).
	self assert: (object atPath: #(foo)) equals: 42.
	self assert: (object atPath: #(foo)) equals: object foo.
	self assert: (object atPath: #(foo)) equals: (object at: #foo).
	self assert: (object atPath: #()) equals: object.
	
! !

!NeoJSONObjectTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/26/2017 16:17'!
testAtPathPut
	| object |
	object := NeoJSONObject new.
	object atPath: #(one test) put: 42.
	self assert: (object atPath: #(one test)) equals: 42.
	self assert: object one test equals: 42.
	object atPath: #(one two three) put: #foo.
	self assert: (object atPath: #(one two three)) equals: #foo.
	self assert: object one two three equals: #foo.
	object atPath: #(one two threeBis) put: #bar.
	self assert: (object atPath: #(one two threeBis)) equals: #bar.
	self assert: object one two threeBis equals: #bar.
	
	self assert: object foo isNil.
	self assert: (object atPath: #() put: 1) equals: object.
	self assert: (object atPath: #(foo) put: 1) equals: 1.
	self assert: (object atPath: #(foo)) equals: 1.
	self assert: object foo equals: 1
	 
! !

!NeoJSONObjectTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/8/2014 12:39'!
testCRUD
	| object |
	object := NeoJSONObject new.
	self assert: object isEmpty.
	self assert: object foo equals: nil.
	object foo: 123.
	self assert: object foo equals: 123.
	self assert: object printString equals: '{"foo":123}'.
	object name: 'test'.
	object value: Float pi.
	self assert: object name equals: 'test'.
	self assert: (object value closeTo: Float pi).
	self assert: object keys asSet equals: #(foo name value) asSet.
	object removeKey: #foo.
	self assert: object foo equals: nil.
	object removeAll.
	self assert: object isEmpty! !

!NeoJSONObjectTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 11:13'!
testJSON
	| data  json object |
	data := NeoJSONObject newFrom: (Array with: #foo->1 with: #bar->2 with: #str->'Str!!' with: #bool->true with: #flt->1.5).
	json := NeoJSONWriter toString: data .
	object := NeoJSONObject fromString: json.
	self assert: object class equals: NeoJSONObject.
	self assert: object equals: data.
	json := (NeoJSONWriter toString: object).
	self assert: (NeoJSONObject fromString: json) equals: object! !

TestCase subclass: #NeoJSONReaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONReaderTests commentStamp: 'SvenVanCaekenberghe 6/2/2012 14:02' prior: 0!
I am NeoJSONReaderTests, a test suite for NeoJSONReader.!

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:03'!
testAllowNil
	self 
		should: [ 
			(NeoJSONReader on: 'null' readStream)
				mapInstVarsFor: Point;
				nextAs: Point ] 
		raise: NeoJSONParseError.
	self
		assert: ((NeoJSONReader on: 'null' readStream)
						mapInstVarsFor: Point;
						for: Point do: [ :mapping | mapping allowNil ];
						nextAs: Point)
		equals: nil.
	self
		assert: ((NeoJSONReader on: '[ { "x" : 1, "y" : 2 }, null, { "x" : 3, "y" : -1 } ]' readStream)
						mapInstVarsFor: Point;
						for: Point do: [ :mapping | mapping allowNil ];
						for: #ArrayOfPoints customDo: [ :mapping | mapping listOfElementSchema: Point ];
						nextAs: #ArrayOfPoints)
		equals: (Array with: 1 @ 2 with: nil with: 3 @ -1).! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 15:28'!
testArrayOfPoints
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: Point do: [ :mapping |
				mapping mapInstVars: #(x y) ];
			for: #ArrayOfPoints customDo: [ :mapping |
				mapping listOfElementSchema: Point ];
			nextAs: #ArrayOfPoints ].
	self assert: (reader value: '[ { "x":1, "y":2 }, { "x":-1, "y":-2 } ]') equals: (Array with: 1@2 with: -1@ -2) . 
	self assert: (reader value: '[ ]') equals: #().
	self assert: (reader value: '[ { } ]') equals: (Array with: Point new)! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 15:29'!
testArrayOfPointsUsingNextListAs
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			mapInstVarsFor: Point;
			nextListAs: Point ].
	self assert: (reader value: '[ { "x":1, "y":2 }, { "y":-2, "x":-1 } ]') equals: (Array with: 1@2 with: -1@ -2) . 
	self assert: (reader value: '[ ]') equals: #().
	self assert: (reader value: '[ { } ]') equals: (Array with: Point new) ! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 13:18'!
testAssociation
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: Association do: [ :mapping |
				mapping mapAccessors: #(key value) ]; 
			nextAs: Association ].
	self assert: (reader value: '{"key":"foo","value":42}') equals: 'foo' -> 42. 
	self assert: (reader value: '{}') equals: Association new! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/13/2012 14:44'!
testBooleans
	self assert: (NeoJSONReader on: 'true' readStream) next equals: true.
	self assert: (NeoJSONReader on: 'false' readStream) next equals: false.
	self should: [ (NeoJSONReader on: 'tru' readStream) next ] raise: NeoJSONParseError.
	self should: [ (NeoJSONReader on: 'falsE' readStream) next ] raise: NeoJSONParseError.! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 12:32'!
testByteArray
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: ByteArray customDo: [ :mapping |
				mapping listOfType: ByteArray ];
			nextAs: ByteArray ].
	self assert: (reader value: '[ 1, 2, 3 ]') equals: #[1 2 3].
	self assert: (reader value: '[ ]') equals: ByteArray new! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 2/6/2018 09:14'!
testDate
	| reader |
	reader := [ :input | 
		(NeoJSONReader on: input readStream) 
			for: Date customDo: [ :mapping |
				mapping decoder: [ :string | Date fromString: string ] ];
			nextAs: Date ].
	self assert: (reader value: ' "2012-06-08" ') equals: (Date fromString: '2012-06-08') ! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 12:02'!
testDateAndTime 
	| reader |
	reader := [ :input | 
		(NeoJSONReader on: input readStream) 
			for: DateAndTime customDo: [ :mapping |
				mapping decoder: [ :string | DateAndTime fromString: string ] ];
			nextAs: DateAndTime ].
	self 
		assert: (reader value: ' "2012-02-14T16:40:18+01:00" ') 
		equals: (DateAndTime year: 2012 month: 2 day: 14 hour: 16 minute: 40 second: 18 offset: 1 hour)! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 21:49'!
testFloats
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: ((reader value: '123.0') closeTo: 123.0).
	self assert: ((reader value: '-123.0') closeTo: -123.0).
	self assert: ((reader value: '123.0e0') closeTo: 123.0).
	self assert: ((reader value: '123e6') closeTo: 123e6).
	self assert: ((reader value: '123e+6') closeTo: 123e6).
	self assert: ((reader value: '123e-6') closeTo: 123e-6).
	self assert: ((reader value: '123E6') closeTo: 123e6).
	self assert: ((reader value: '123E+6') closeTo: 123e6).
	self assert: ((reader value: '123E-6') closeTo: 123e-6).
	self assert: ((reader value: '-123e6') closeTo: -123e6).
	self assert: ((reader value: '-123e-6') closeTo: -123e-6).
	self assert: ((reader value: '-123E6') closeTo: -123e6).
	self assert: ((reader value: '-123E-6') closeTo: -123e-6).
	self assert: ((reader value: '3.14159') closeTo: Float pi).
	self assert: ((reader value: '0.123e6') closeTo: 0.123e6).
	self assert: ((reader value: '0.123e-6') closeTo: 0.123e-6).
	self assert: ((reader value: '-0.123E6') closeTo: -0.123e6).
	self assert: ((reader value: '-0.123E-6') closeTo: -0.123e-6).
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 16:08'!
testIntegers
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '123') equals: 123.
	self assert: (reader value: '00123') equals: 123.
	self assert: (reader value: '-123') equals: -123.
	self assert: (reader value: '-0123') equals: -123.
	self assert: (reader value: '0') equals: 0.
	self assert: (reader value: '-0') equals: 0.
	self assert: (reader value: '00') equals: 0.
	self assert: (reader value: ' 123	') equals: 123.
	self should: [ reader value: '+1' ] raise: NeoJSONParseError.
	self should: [ reader value: ' - ' ] raise: NeoJSONParseError.
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 16:16'!
testLists
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '[1,2,3]') equals: #(1 2 3).
	self assert: (reader value: '[]') equals: #().
	self assert: (reader value: ' [ 1 , 2 , 3 ] ') equals: #(1 2 3).
	self assert: (reader value: ' [ ] ') equals: #().
	self should: [ reader value: '[1,]' ] raise: NeoJSONParseError. 
	self should: [ reader value: '[' ] raise: NeoJSONParseError.
	self should: [ reader value: ']' ] raise: NeoJSONParseError.
	self should: [ reader value: '[}' ] raise: NeoJSONParseError! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 16:16'!
testMaps
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '{"x":1,"y":2}') equals: (Dictionary newFromPairs: #( 'x' 1 'y' 2)).
	self assert: (reader value: '{}') equals: Dictionary new.
	self assert: (reader value: ' { "x" : 1 , "y" : 2 } ' ) equals: (Dictionary newFromPairs: #( 'x' 1 'y' 2)).
	self assert: (reader value: ' { } ') equals: Dictionary new.
	self should: [ reader value: '{"foo":}' ] raise: NeoJSONParseError. 
	self should: [ reader value: '{"foo":9,}' ] raise: NeoJSONParseError. 
	self should: [ reader value: '{9:true,}' ] raise: NeoJSONParseError. 
	self should: [ reader value: '{' ] raise: NeoJSONParseError.
	self should: [ reader value: '}' ] raise: NeoJSONParseError.
	self should: [ reader value: '{]' ] raise: NeoJSONParseError! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/24/2014 11:58'!
testNextStringAsHex
	| string |
	string := (NeoJSONReader on: '"466F6F2062617221"' readStream)
		for: #StringAsHex
			customDo: [ :mapping | 
					mapping
						encoder: [ :x | x asByteArray hex asUppercase ];
						decoder: [ :x | (ByteArray readHexFrom: x) asString ] ];
		nextAs: #StringAsHex.
	self assert: string equals: 'Foo bar!!'! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 11:58'!
testNonBMPCharacterDecoding
	"Characters not in the Basic Multilingual Plane are encoded as a UTF-16 surrogate pair"

	| string object |
	string := 16r1D11E asCharacter asString. "MUSICAL SYMBOL G CLEF"
	object := (NeoJSONReader fromString: '"\uD834\uDD1E"').
	self assert: object equals: string! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/13/2012 14:29'!
testNull
	self assert: (NeoJSONReader on: 'null' readStream) next equals: nil.
	self assert: (NeoJSONReader on: '	null ' readStream) next equals: nil.
	self should: [ (NeoJSONReader on: '	nil ' readStream) next ] raise: NeoJSONParseError.! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/13/2012 10:18'!
testOrderedCollection
	| json list |
	json := '[ 1, 2, 3 ]'.
	list := (NeoJSONReader on: json readStream) 
		listClass: OrderedCollection;
		next.
	self 
		assert: list 
		equals: (OrderedCollection with: 1 with: 2 with: 3).
	self 
		assert: list class 
		equals: OrderedCollection! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:04'!
testOrderedCollectionOfPoints
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: Point do: [ :mapping |
				mapping mapInstVars: #(x y) ];
			for: #OrderedCollectionOfPoints customDo: [ :mapping |
				mapping listOfType: OrderedCollection andElementSchema: Point ];
			nextAs: #OrderedCollectionOfPoints ].
	self assert: (reader value: '[ { "x":1, "y":2 }, { "x":-1, "y":-2 } ]') equals: (Array with: 1@2 with: -1@ -2) asOrderedCollection. 
	self assert: (reader value: '[ ]') equals: OrderedCollection new.
	self assert: (reader value: '[ { } ]') equals: (OrderedCollection with: Point new)! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/29/2017 11:00'!
testParseErrors
	self should: [ NeoJSONReader fromString: 'zero' ] raise: NeoJSONParseError.
	self should: [ NeoJSONReader fromString: '#' ] raise: NeoJSONParseError.
	self should: [ NeoJSONReader fromString: '@' ] raise: NeoJSONParseError.
	
	self should: [ (NeoJSONReader on: '[1,2,3]true' readStream) next; failIfNotAtEnd ] raise: NeoJSONParseError.
	self should: [ (NeoJSONReader on: '{"foo":1}"extra"' readStream) next; failIfNotAtEnd ] raise: NeoJSONParseError.
	self should: [ (NeoJSONReader on: '[]"extra"' readStream) next; failIfNotAtEnd ] raise: NeoJSONParseError.! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'MarianoMartinezPeck 1/25/2018 09:37'!
testPoint
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) nextAs: Point ].
	self should: [ reader value: '{"x":1,"y":2}' ] raise: NeoJSONMappingNotFound! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 14:04'!
testPoint1
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: Point do: [ :mapping |
				mapping mapProperty: 'x' getter: [ :point | point x ] setter: [ :point :value | point setX: value setY: point y ].
				mapping mapProperty: 'y' getter: [ :point | point y ] setter: [ :point :value | point setX: point x setY: value ]. ];
			nextAs: Point ].
	self assert: (reader value: '{"x":1,"y":2}') equals: 1@2. 
	self assert: (reader value: '{}') equals: Point new! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/7/2012 15:23'!
testPoint2
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			mapAllInstVarsFor: Point;
			nextAs: Point ].
	self assert: (reader value: '{"x":1,"y":2}') equals: 1@2. 
	self assert: (reader value: '{}') equals: Point new! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 14:04'!
testPoint3
	| reader |
	reader := [ :string | 
		(NeoJSONReader on: string readStream) 
			for: Point do: [ :mapping |
				mapping mapInstVars: #(x y) ];
			nextAs: Point ].
	self assert: (reader value: '{"x":1,"y":2}') equals: 1@2. 
	self assert: (reader value: '{}') equals: Point new! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/24/2014 10:43'!
testPointUsingAsString
	| point |
	point := (NeoJSONReader on: '{"x":"1","y":"2"}' readStream)
		for: Point
			do: [ :mapping | 
					(mapping 
						mapProperty: #x 
						getter: [ :object | object x ] 
						setter: [ :object :value | object setX: value setY: object y ])
						valueSchema: #AsString.
					(mapping
						mapProperty: #y
						getter: [ :object | object y ]
						setter: [ :object :value | object setX: object x setY: value ]) 
						valueSchema: #AsString ];
		for: #AsString
			customDo: [ :mapping | 
					mapping
						encoder: [ :x | x asString ];
						decoder: [ :x | x asNumber ] ];
		nextAs: Point.
	self assert: point equals: 1 @ 2! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/13/2012 10:00'!
testPropertyNamesAsSymbols
	| json map |
	json := '{ "x" : true, "y" : false, "key-1" : 1 }'.
	map := (NeoJSONReader on: json readStream) 
		propertyNamesAsSymbols: true; 
		next.
	self 
		assert: map 
		equals: (Dictionary new at: #x put: true; at: #y put: false; at: #'key-1' put: 1; yourself).
	self assert: (map keys allSatisfy: [ :each | each isSymbol ])! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 2/1/2018 10:07'!
testSmallDictionary
	| json map smallDictionaryClass |
	json := '{ "x" : true, "y" : false, "key-1" : 1 }'.
	smallDictionaryClass := Smalltalk at: #SmallDictionary ifAbsent: [ Dictionary ].
	map := (NeoJSONReader on: json readStream) 
		mapClass: smallDictionaryClass;
		next.
	self 
		assert: map 
		equals: (smallDictionaryClass new at: 'x' put: true; at: 'y' put: false; at: 'key-1' put: 1; yourself).
	self 
		assert: map class 
		equals: smallDictionaryClass! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:01'!
testStrings
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '"foo"') equals: 'foo'.
	self assert: (reader value: '""') equals: ''.
	self assert: (reader value: '"Foo BAR"') equals: 'Foo BAR'.
	self assert: (reader value: '" foo "') equals: ' foo '.
	self assert: (reader value: '"\u00E9l\u00E8ve en Fran\u00E7ais"') equals: 'élève en Français'.
	self 
		assert: (reader value: '"\"\\\/\t\r\n\f\b"') 
		equals: (String withAll: ( OrderedCollection new 
			add: $";
			add: $\; 
			add: $/;
			add: Character tab;
			add: Character cr;
			add: Character lf;
			add: Character newPage; 
			add: Character backspace;
			yourself )).
	self should: [ reader value: '"open' ] raise: NeoJSONParseError.
	self should: [ reader value: '"\uAA"' ] raise: NeoJSONParseError.
	self should: [ reader value: '"\uZZ"' ] raise: NeoJSONParseError.
	self should: [ reader value: '"\x"' ] raise: NeoJSONParseError.
	self should: [ reader value: ' " ' ] raise: NeoJSONParseError.
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 12:02'!
testSymbol
	| reader |
	reader := [ :input | 
		(NeoJSONReader on: input readStream) 
			for: Symbol customDo: [ :mapping |
				mapping decoder: [ :string | string asSymbol ] ];
			nextAs: Symbol ].
	self assert: (reader value: ' "foo" ') equals: #foo.
	self assert: (reader value: ' "FOO-1" ') equals: #'FOO-1'! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'MarianoMartinezPeck 2/12/2018 09:44'!
testTime
	| reader |
	reader := [ :input | 
		(NeoJSONReader on: input readStream) 
			for: Time customDo: [ :mapping |
				mapping decoder: [ :string | Time fromString: string ] ];
			nextAs: Time ].
	self assert: (reader value: ' "14:30:08" ') equals: (Time fromString: '14:30:08')! !

TestCase subclass: #NeoJSONWriteReadTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadTests commentStamp: 'SvenVanCaekenberghe 6/4/2012 15:34' prior: 0!
I am NeoJSONWriteReadTests.!

NeoJSONWriteReadTests subclass: #NeoJSONWriteReadAsciiOnlyTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadAsciiOnlyTests commentStamp: 'SvenVanCaekenberghe 11/24/2016 10:07' prior: 0!
I am NeoJSONWriteReadAsciiOnlyTests.!

!NeoJSONWriteReadAsciiOnlyTests methodsFor: 'private' stamp: 'MarianoMartinezPeck 2/13/2018 09:23'!
writeRead: object
	| json |
	json := self stringStreamContents: [ :stream |
		(NeoJSONWriter on: stream) asciiOnly: true; nextPut: object ].
	^ (NeoJSONReader on: json readStream) next ! !

NeoJSONWriteReadTests subclass: #NeoJSONWriteReadMockStreamTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadMockStreamTests commentStamp: 'SvenVanCaekenberghe 11/24/2016 10:07' prior: 0!
I am NeoJSONWriteReadMockStreamTests.!

!NeoJSONWriteReadMockStreamTests methodsFor: 'private' stamp: 'MarianoMartinezPeck 2/13/2018 09:23'!
writeRead: object
	| json |
	json := self stringStreamContents: [ :stream |
		(NeoJSONWriter on: (NeoJSONMockStream on: stream))
			prettyPrint: true;
			nextPut: object ].
	^ (NeoJSONReader on: (NeoJSONMockStream on: json readStream)) next ! !

NeoJSONWriteReadTests subclass: #NeoJSONWriteReadPrettyPrintedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadPrettyPrintedTests commentStamp: 'SvenVanCaekenberghe 6/5/2012 09:50' prior: 0!
I am NeoJSONWriteReadPrettyPrintedTests.!

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'testing' stamp: 'marianopeck 2/1/2018 09:39'!
testDictionaryOfObject2
	| data json reader |
	data := Dictionary new.
	data 
		at: 'one' put: NeoJSONTestObject2 example1;
		at: 'two' put: NeoJSONTestObject2 example1.
	json := NeoJSONWriter toStringPretty: data.
	reader := NeoJSONReader on: json readStream.
	reader for: #DictionaryOfObject2 customDo: [ :mapping |
		mapping mapWithValueSchema: NeoJSONTestObject2 ].  
	self 
		assert: (reader nextAs: #DictionaryOfObject2)
		equals: data! !

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:49'!
testObject1
	| json |
	json := NeoJSONWriter toStringPretty: NeoJSONTestObject1 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject1)
		equals: NeoJSONTestObject1 example1! !

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/30/2013 11:05'!
testObject2
	| json |
	json := NeoJSONWriter toStringPretty: NeoJSONTestObject2 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject2)
		equals: NeoJSONTestObject2 example1! !

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/22/2014 10:59'!
testObject3
	| json |
	json := NeoJSONWriter toStringPretty: NeoJSONTestObject3 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject3)
		equals: NeoJSONTestObject3 example1! !

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'private' stamp: 'MarianoMartinezPeck 2/13/2018 09:22'!
writeRead: object
	| json |
	json := self stringStreamContents: [ :stream |
		(NeoJSONWriter on: stream) 
			prettyPrint: true;
			nextPut: object ].
	^ (NeoJSONReader on: json readStream) next ! !

!NeoJSONWriteReadTests methodsFor: 'private' stamp: 'MarianoMartinezPeck 2/13/2018 09:24'!
stringStreamContents: aBlock
	^ String streamContents: aBlock! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 2/1/2018 09:39'!
testDictionaryOfObject2
	| data json reader |
	data := Dictionary new.
	data 
		at: 'one' put: NeoJSONTestObject2 example1;
		at: 'two' put: NeoJSONTestObject2 example1.
	json := NeoJSONWriter toString: data.
	reader := NeoJSONReader on: json readStream.
	reader for: #DictionaryOfObject2 customDo: [ :mapping |
		mapping mapWithValueSchema: NeoJSONTestObject2 ].  
	self 
		assert: (reader nextAs: #DictionaryOfObject2)
		equals: data! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:16'!
testFloats
	| objects |
	objects := OrderedCollection new add: Float pi; add: 1.0; add: -1.0; add: 0.0; add: 1.5e6; add: -1.5e6; add: 1.5e-6; add: -1.5e-6; yourself.
	(self writeRead: objects) 
		with: objects 
		do: [ :first :second | self assert: (first closeTo: second) ]! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:17'!
testIntegers
	| objects |
	objects := Array with: 1 with: -1 with: 0 with: 123467890 with: -1234567890.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:18'!
testLists
	| objects |
	objects := Array with: #(1 2 3) with: #() with: #( 'foo' 'bar').
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:19'!
testMaps
	| objects |
	objects := Array 
		with: Dictionary new
		with: (Dictionary with: 'x' -> 1)
		with: (Dictionary newFromPairs: #( 'foo' true 'bar' false 'zero' 0 'null' nil 'string' 'Hello World!!' )).
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:48'!
testObject1
	| json |
	json := NeoJSONWriter toString: NeoJSONTestObject1 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject1)
		equals: NeoJSONTestObject1 example1! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/30/2013 10:12'!
testObject2
	| json |
	json := NeoJSONWriter toString: NeoJSONTestObject2 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject2)
		equals: NeoJSONTestObject2 example1! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/22/2014 10:14'!
testObject3
	| json |
	json := NeoJSONWriter toString: NeoJSONTestObject3 example1.
	self 
		assert: (NeoJSONReader fromString: json as: NeoJSONTestObject3)
		equals: NeoJSONTestObject3 example1! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:20'!
testSpecials
	| objects |
	objects := Array with: true with: false with: nil.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 12:00'!
testStrings
	| objects |
	objects := OrderedCollection new
		add: 'foo'; add: 'Foo BAR'; add: ''; add: '	\\'''; 
		add: 'élève en Français';
		add: (Character codePoint: 12354) asString; "HIRAGANA LETTER A" 
		yourself;
		asArray.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'private' stamp: 'MarianoMartinezPeck 2/13/2018 09:23'!
writeRead: object
	| json |
	json := self stringStreamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: object ].
	^ (NeoJSONReader on: json readStream) next ! !

TestCase subclass: #NeoJSONWriterTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriterTests commentStamp: 'SvenVanCaekenberghe 6/2/2012 22:20' prior: 0!
I am NeoJSONWriterTests, a test suite for NeoJSONWriter.!

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 12:59'!
testAsciiOnlyStrings
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				asciiOnly: true;
				nextPut: object ] ].
	self assert: (writer value: 'élève français') equals: '"\u00E9l\u00E8ve fran\u00E7ais"'.
	self assert: (writer value: 'foo') equals: '"foo"'.
	self assert: (writer value: 'Foo BAR') equals: '"Foo BAR"'.
	self assert: (writer value: '') equals: '""'.
	self
		assert: (writer value: (String withAll: ( OrderedCollection new 
			add: $";
			add: $\; 
			add: $/;
			add: Character tab;
			add: Character cr;
			add: Character lf;
			add: Character newPage; 
			add: Character backspace;
			yourself )))
		equals: '"\"\\/\t\r\n\f\b"'

! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 13:19'!
testAssociation
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: Association do: [ :mapping |
					mapping mapAccessors: #(key value) ]; 
				nextPut: object ] ].
	self assert: (writer value: 'foo' -> 42) equals: '{"key":"foo","value":42}'.
	self assert: (writer value: Association new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:56'!
testBooleans
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: true) equals: 'true'.
	self assert: (writer value: false) equals: 'false'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 21:54'!
testByteArray
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: #[1 2 3]) equals: '[1,2,3]'.
	self assert: (writer value: #[]) equals: '[]'
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'marianopeck 2/6/2018 09:14'!
testDate
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: Date customDo: [ :mapping |
					mapping encoder: [ :date | date yyyymmdd ] ];
				nextPut: object ] ].
	self assert: (writer value: (Date fromString: '2012-06-08')) equals: '"2012-06-08"'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/12/2012 12:02'!
testDateAndTime 
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: DateAndTime customDo: [ :mapping |
					mapping encoder: [ :dateAndTime | dateAndTime printString ] ];
				nextPut: object ] ].
	self 
		assert: (writer value: (DateAndTime year: 2012 month: 2 day: 14 hour: 16 minute: 40 second: 18 offset: 1 hour)) 
		equals: '"2012-02-14T16:40:18+01:00"'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:24'!
testFloats
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 123.0) equals: '123.0'.
	self assert: (writer value: -123.0) equals: '-123.0'.
	self assert: (writer value: 0.0) equals: '0.0'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:24'!
testIntegers
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 123) equals: '123'.
	self assert: (writer value: -123) equals: '-123'.
	self assert: (writer value: 0) equals: '0'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:39'!
testLists
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: #(1 2 3)) equals: '[1,2,3]'.
	self assert: (writer value: #()) equals: '[]'.
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/4/2015 14:02'!
testListsExtra
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	#( OrderedCollection LinkedList ByteArray IntegerArray SortedCollection ) do: [ :each |
		Smalltalk at: each ifPresent: [ :collectionClass |
			self assert: (writer value: (collectionClass withAll: #(1 2 3))) equals: '[1,2,3]'.
			self assert: (writer value: collectionClass new) equals: '[]' ] ]
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'marianopeck 2/6/2018 08:37'!
testMaps
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: ((writer value: (Dictionary newFromPairs: #( 'x' 1 'y' 2))) includesSubstring: '"x":1').
	self assert: ((writer value: (Dictionary newFromPairs: #( 'x' 1 'y' 2))) includesSubstring: '"y":2').
	self assert: (writer value: Dictionary new) equals: '{}'.
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/26/2015 14:05'!
testMapsExtra
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: (IdentityDictionary newFromPairs: #( 'x' 1))) equals: '{"x":1}'.
	self assert: (writer value: IdentityDictionary new) equals: '{}'.
	#( SmallDictionary OrderedDictionary OrderedIdentityDictionary ) do: [ :each |
		| dictionaryClass |
		dictionaryClass := Smalltalk at: each ifAbsent: [ nil ].
		dictionaryClass ifNotNil: [	
			self assert: (writer value: (dictionaryClass new at: 'x' put: 1; at: 'y' put: 2; yourself)) equals: '{"x":1,"y":2}'.
			self assert: (writer value: dictionaryClass new) equals: '{}'] ]
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/24/2014 11:57'!
testNextPutStringAsHex
	| output |
	output := String
		streamContents: [ :out | 
			(NeoJSONWriter on: out)
				for: #StringAsHex
					customDo: [ :mapping | 
							mapping
								encoder: [ :x | x asByteArray hex asUppercase ];
								decoder: [ :x | (ByteArray readHexFrom: x) asString ] ];
				nextPut: 'Foo bar!!' as: #StringAsHex ].
	self assert: output equals: '"466F6F2062617221"'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'marianopeck 1/31/2018 12:00'!
testNonBMPCharacterEncoding
	"Characters not in the Basic Multilingual Plane are encoded as a UTF-16 surrogate pair"

	| string json |
	string := 16r1D11E asCharacter asString. "MUSICAL SYMBOL G CLEF"
	json := String streamContents: [ :out |
		(NeoJSONWriter on: out) asciiOnly: true; nextPut: string ].
	self assert: json equals: '"\uD834\uDD1E"'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:23'!
testNull
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: nil) equals: 'null'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'MarianoMartinezPeck 1/25/2018 09:37'!
testPoint
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self should: [ writer value: 1@2 ] raise: NeoJSONMappingNotFound! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 14:04'!
testPoint1
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: Point do: [ :mapping |
					mapping mapProperty: 'x' getter: [ :point | point x ] setter: [ :point :value | point x: value ].
					mapping mapProperty: 'y' getter: [ :point | point y ] setter: [ :point :value | point y: value ]. ];
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 17:29'!
testPoint2
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				mapAllInstVarsFor: Point;
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 14:04'!
testPoint3
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: Point do: [ :mapping |
					mapping mapInstVars: #(x y) ];
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/24/2014 10:41'!
testPointUsingAsString
	| output |
	output := String
		streamContents: [ :out | 
			(NeoJSONWriter on: out)
				for: Point
					do: [ :mapping | 
							(mapping 
								mapProperty: #x 
								getter: [ :object | object x ] 
								setter: [ :object :value | object setX: value setY: object y ])
								valueSchema: #AsString.
							(mapping
								mapProperty: #y
								getter: [ :object | object y ]
								setter: [ :object :value | object setX: object x setY: value ]) 
								valueSchema: #AsString ];
				for: #AsString
					customDo: [ :mapping | 
							mapping
								encoder: [ :x | x asString ];
								decoder: [ :x | x asNumber ] ];
				nextPut: 1 @ 2 ].
	self assert: output equals: '{"x":"1","y":"2"}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/28/2014 20:49'!
testPointWriteNil
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				writeNil: true;
				mapAllInstVarsFor: Point;
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{"x":null,"y":null}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 7/1/2016 23:50'!
testPreservePropertyOrder
	| writer testObject |
	(testObject := NeoJSONTestObject2 new)
		id: 123;
		width: 100;
		height: 50;
		data: 'test'.
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: NeoJSONTestObject2 do: [ :mapping | mapping mapInstVars: #(id width height data) ];
				nextPut: object ] ].
	self assert: (writer value: testObject) equals: '{"id":123,"width":100,"height":50,"data":"test"}'.
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: NeoJSONTestObject2 do: [ :mapping | mapping mapInstVars: #(width height data id) ];
				nextPut: object ] ].
	self assert: (writer value: testObject) equals: '{"width":100,"height":50,"data":"test","id":123}'.
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: NeoJSONTestObject2 do: [ :mapping | mapping mapInstVars: #(data id height width) ];
				nextPut: object ] ].
	self assert: (writer value: testObject) equals: '{"data":"test","id":123,"height":50,"width":100}'.
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'marianopeck 1/30/2018 13:00'!
testStrings
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 'foo') equals: '"foo"'.
	self assert: (writer value: 'Foo BAR') equals: '"Foo BAR"'.
	self assert: (writer value: '') equals: '""'.
	self
		assert: (writer value: (String withAll: ( OrderedCollection new 
			add: $";
			add: $\; 
			add: $/;
			add: Character tab;
			add: Character cr;
			add: Character lf;
			add: Character newPage; 
			add: Character backspace;
			yourself )))
		equals: '"\"\\/\t\r\n\f\b"'


! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2012 21:56'!
testSymbol
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: #foo) equals: '"foo"'.
	self assert: (writer value: #'FOO-1') equals: '"FOO-1"'
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'MarianoMartinezPeck 2/12/2018 09:45'!
testTime
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				for: Time customDo: [ :mapping |
					mapping encoder: [ :time | time printString ] ];
				nextPut: object ] ].
	self assert: ((writer value: (Time fromString: '14:30:08')) includesSubstring: '2:30:08')! !
